
%% Define parameters
clear all; clc; close all;
L_SI        = 4000; % length of well, [m]
rho_mud     = 1100; % mud weight bentonite slurry, [kg/m^3]
rho_mud_conv = 1300;
rho_cem     = 2000; % Portland cement, [kg/m^3]%Friksjons dominert/ hydrostatisk dominert
rho_spacer  = 1600;
Cd_o        = 13.375; % casing outer diameter, [in]
Sd_i        = 5; % stinger inner diameter, [in]
Od          = 17.5; % Openhole diameter, [in]
PV_spacer   = 62;
YP_SI_spacer= 6.3;
tau_y_spacer= 0.75;
PV_mud      = 30; % plastic viscosity mud, [cP]
PV_cem      = 334.2; % plastic viscosity cement, [cP]
YP_SI_mud   = 13.5; %yield point mud, [Pa]
YP_SI_cem   = 27; %yield point cement, [Pa]
tau_y_mud   = 0.75; % yield stress mud, [lb/100ft^2]
tau_y_cem   = 0.75; % yield stress cement, [lb/100ft^2]
g           = 9.81; % gravity constant, [m/s^2]
in2m        = 0.0254; % multiplier from in to m
lpm2m3s     = 1/60000;% multiplier from lpm to m3/s
bar2Pa      = 1e5;% multiplier from bar to Pa
B_mud       = 1.5*10^4 * bar2Pa; % isothermal bulk modulus mud [Pa]
B_cem       = 40*10^4 * bar2Pa; % isothermal bulk modulus mud [Pa]
B_spacer    = 1.5*10^4*bar2Pa;
Ap          = pi*(Sd_i*in2m/2)^2;%m2
Aa          = pi*((Od*in2m/2)^2-(Cd_o*in2m/2)^2);
V_tp        = Ap*L_SI;%m3 total volume pipe
V_ta        = Aa*L_SI;%m3 total volume annulus
Vst1        = 10;%m3 % 60 bbl pre-spacer
Vst2        = 1;%m3 % 5 bbl flush-spacer

t_m  = 500; %time pumping mud before cement 
dt   = 0.01;%s
T    = 7200;%s
t    = dt:dt:T;
N    = length(t);
qp_mud  = 1000*lpm2m3s;%m3/s
qp_cem = 500*lpm2m3s; %m3/s
qp_mud_dis = 1500*lpm2m3s;%m3/s
P0   = 0e5; % Gauge Pressure Atmospheric pressure = 0 bar
Pc0  = (rho_mud_conv-rho_mud)*g*L_SI; % Initial applied back pressure
Lmp0 = L_SI;
Lma0 = Lmp0;
u0   = 0.2;
Lcem= 300;
Vct = Lcem*Aa; %total cement to be pumped [m3]
Cv   = qp_mud/u0/sqrt(Pc0/rho_mud); % Choke constant based on initial values

qp = zeros(1,N);% ones(1,N)*qp0;
Ga = zeros(1,N);%ones(1,N)*rho_mud*g*L_SI;%Pa
Gp = zeros(1,N);%ones(1,N)*rho_mud*g*L_SI;%Pa
Mp = zeros(1,N);%ones(1,N)*rho_mud*L_SI/((pi*(Sd_i^2)/4)*in2m^2);%kg/m4
Ma = zeros(1,N);%ones(1,N)*rho_mud*L_SI/((pi*((Od^2)-(Cd_o^2))/4)*in2m^2);%kg/m4

qb      = zeros(1,N);
qc      = zeros(1,N);  
qavg_pipe = zeros(1,N);
Fp      = zeros(1,N); 
Fa      = zeros(1,N); 
Pp      = zeros(1,N); 
Pc      = zeros(1,N); 
Pdhp    = zeros(1,N);
Pdha    = zeros(1,N);
deltaP  = zeros(1,N);
Lcp     = zeros(1,N);
Lcpf    = zeros(1,N);
Lca     = zeros(1,N);
Lma     = zeros(1,N);
Lmp     = zeros(1,N);
Lsa = zeros(1,N);
Lsp     = zeros(1,N);
Lspf = zeros(1,N);
Lsp1= zeros(1,N);
Lsp2= zeros(1,N);
hd      = zeros(1,N);
Vcp     = zeros(1,N);%cement in pipe
Vcp_c   = zeros(1,N);%cement in pipe before compressed
Vmp1    = zeros(1,N);%mud ahead of cement
Vmp2    = zeros(1,N);%displaced mud behind cement
Vmp_c1  = zeros(1,N);%displaced mud in pipe before compressed
Vmp_c2  = zeros(1,N);%displacing mud in pipe before compressed
Vma_c   = zeros(1,N);%mud in annulus before compressed
Vca_c   = zeros(1,N);%cement in annulus before compressed
Vca     = zeros(1,N);%cement in annulus
Vma     = zeros(1,N);%mud in annulus
Vaa     = zeros(1,N);%air gap in annulus

Vsp1     = zeros(1,N);%cement in pipe
Vsp_c1   = zeros(1,N);%cement in pipe before compressed
Vsp2     = zeros(1,N);%cement in pipe
Vsp_c2   = zeros(1,N);%cement in pipe before compressed
Vsa      = zeros(1,N);
Vsa_c = zeros(1,N);

e       = zeros(1,N);
eI      = zeros(1,N);
u       = zeros(1,N);%u0*ones(N,1);
ref     = zeros(1,N);%Desired downhole pressure
statec  = zeros(1,N);%0=pump mud, 1=pump cement, 2=displace with mud, 3=mud in annulus, 4=done

%Controller
Kp      = 0.3*1e-5;% 1 bar offset will give 10% change in choke pos
Ti      = 5;%sec
umin    = 0.0;

%% Initial conditions
qp(1)   = qp_mud; %qp0; 
qb(1)   = qp_mud; %qp0; 
qc(1)   = qp_mud; %qp0;
qavg_pipe(1) =qp_mud;
Pc(1)   = Pc0;
Ga(1)   = rho_mud*g*L_SI;%Pa
Gp(1)   = rho_mud*g*L_SI;%Pa
Mp(:)   = rho_mud*L_SI/((pi*(Sd_i^2)/4)*in2m^2);%kg/m4
Ma(:)   = rho_mud*L_SI/((pi*((Od^2)-(Cd_o^2))/4)*in2m^2);%kg/m4

Fp(1)   = unified_friction_pipe(Sd_i,qp_mud,L_SI,PV_mud,YP_SI_mud,tau_y_mud,rho_mud); % friction pressure drop pipe, [Pa] 
Fa(1)   = unified_friction_ann(Cd_o,Od,qp_mud,L_SI,PV_mud,YP_SI_mud,tau_y_mud,rho_mud); % friction pressure drop annulus, [Pa]
Pdhp(1) = Pc(1) + Fa(1) + Ga(1);
Pdha(1) = Pc(1) + Ga(1);
Pp(1)   = Pdha(1) + Fp(1) - Gp(1);
r0      = Pdha(1);
hd(1)   = 0;
ref(1)  = r0;
e(1)    = 0;
eI(1)   = Ti/Kp*(u0 - Kp*e(1));        
u(1)    = u0;
Lma(1)  = Lma0;
Lmp(1)  = Lmp0;
Vmp1(1) = V_tp;
Vma(1)  = V_ta;


%% Flow
alpha_pipe = 0.5;
alpha_ann  = 0;
tramp = 20;%sec
noise = zeros(1,N);%0.02e5*randn(1,N);
headindp  = true;
for k = 2:N
    if t(k) < 50 % bygger opp pumpetrykk
        qp(k) = qp_mud; %qp0;
    elseif (t(k) < 100)
        qp(k) = min(qp_mud*1.2 , qp(k-1)+dt*qp_mud/tramp); %min(qp0*2,qp(k-1)+dt*qp0/tramp);
    elseif t(k) < 150
        qp(k) = min(qp_mud*1.5 ,  qp(k-1)+dt*qp_mud/tramp); %min(qp0,  qp(k-1)+dt*qp0/tramp);;
    elseif t(k) < 250 
        qp(k) = qp_mud*1.5;
    else 
        qp(k) = max(qp_cem, qp(k-1)-dt*qp_mud/tramp); %min(qp0*2,qp(k-1)+dt*qp0/tramp);
    end
        
   %State machine
    if statec(k-1) == 0
        if t(k) <= t_m % pumping mud
            statec(k) = 0;
        else
            statec(k) = 1;
        end
    elseif statec(k-1) == 1 
        if Vsp1(k-1) < Vst1 %pumping spacer
            statec(k) = 1;
        else 
            statec(k) = 2;
        end
    elseif statec(k-1) == 2
        if Vcp(k-1) < Vct %pumping cement
            statec(k) = 2;
        else
            statec(k) = 3;
        end
    elseif statec(k-1) == 3 
        if Vsp2(k-1) < Vst2 %pumping spacer
            statec(k) = 3;
        else 
            statec(k) = 4;
        end
    elseif statec(k-1) == 4 %pumping mud
        if Lspf(k-1) < L_SI % 
            statec(k) = 4;
        else
            statec(k) = 5;
        end
    elseif statec(k-1) == 5
      if Lcpf(k-1) < L_SI % Cement front reach bottom of well
            statec(k) = 5;
      else
          statec(k) = 6;
      end
        elseif statec(k-1) == 6 
        if  Vcp(k-1) > 0.416 % 1.7 n?r qcem = 1000 lpm, n?r qcem = 500 lpm --> 0.416
            statec(k) = 6;
        else
            statec(k) = 7;
            kend      = N;
        end
        elseif statec(k-1) == 7
        statec(k) = 7;
    end
    
    if statec(k) >= 4
                    qp(k) = min(qp_mud_dis, qp(k-1)+dt*qp_mud/tramp);
    end
if statec(k) == 0 % pumping mud  
        Vmp1(k)         = V_tp;
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Vmp_c1(k)       = V_tp;
        Vcp(k)          = 0; 
        Lcp(k)          = Vcp(k)/Ap;
        Lcpf(k)         = Lcp(k); 
        
        Pp(k)           = max(0,Pp(k-1) + dt*(B_mud/V_tp)*(qp(k-1) - qb(k-1)));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qc(k-1)));
    elseif statec(k) == 1 % pumping spacer
        Vmp_c1(k)       = Vmp1(k-1) - dt*(qb(k-1));
        Vsp_c1(k)       = Vsp1(k-1) + dt*(qp(k-1));
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_spacer_pipe1 = Vsp_c1(k)/B_spacer;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k))/(const_mud_pipe1+const_spacer_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsp1(k)          = Vsp_c1(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Lsp1(k)          = Vsp1(k)/Ap;
        Lspf(k)         = Lsp(k) +hd(k-1);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_spacer_pipe2 = Vsp1(k)/B_spacer;
    
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+const_spacer_pipe2));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qc(k-1)));
    elseif statec(k) == 2 % pumping cement
        Vmp_c1(k)       = Vmp1(k-1) - dt*(qb(k-1));
        Vcp_c(k)        = Vcp(k-1) + dt*(qp(k-1));
        Vsp_c1(k)       = Vsp_c1(k-1);
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        const_spacer_pipe1=Vsp_c1(k)/B_spacer;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vcp_c(k)-Vsp_c1(k))/(const_mud_pipe1+const_cem_pipe1+const_spacer_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem); % solving for new compressed volume of cement
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Lcp(k)          = Vcp(k)/Ap;
        Lcpf(k)         = Lcp(k)+hd(k-1);
        Lsp1(k)          = Vsp1(k)/Ap;
        Lspf(k)         = Lcpf(k)+Lsp1(k);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_cem_pipe2 = Vcp(k)/B_cem;
        const_spacer_pipe2=Vsp1(k)/B_spacer;
    
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+const_cem_pipe2+const_spacer_pipe2));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qc(k-1)));
    elseif statec(k) == 3  % Cement er ferdig pumpet, starter aa pumpe spacer 2           
        Vcp_c(k)        = Vcp_c(k-1); 
        Vsp_c1(k)       = Vsp_c1(k-1);
        Vmp_c1(k)       = max(0,Vmp1(k-1) - dt*(qb(k-1))); 
        Vsp_c2(k)       = Vsp2(k-1) + dt*(qp(k-1));
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k)-Vsp_c2(k)-Vcp_c(k))/(const_mud_pipe1+const_spacer1_pipe1+ const_spacer2_pipe1+ const_cem_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud);
        Vsp2 (k)        = Vsp_c2(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of mud
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem);% solving for new compressed volume of cement
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)+Vmp2(k))/Ap+hd(k-1); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_mud2_pipe2=Vmp2(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_mud2_pipe2+ const_spacer1_pipe2+const_spacer2_pipe2+const_cem_pipe2));
        Pdha(k)         = Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1)-qc(k-1));
        
    elseif statec(k) == 4 %cement ferdig pumpet, spacer ferdig pumpet, pumping mud! 
        
        Vcp_c(k)        = Vcp_c(k-1); 
        Vsp_c1(k)       = Vsp_c1(k-1);
        Vsp_c2(k)       = Vsp_c2(k-1);
        Vmp_c1(k)       = max(0,Vmp1(k-1) - dt*(qb(k-1))); 
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_mud2_pipe1=Vmp_c2(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k)-Vsp_c2(k)-Vcp_c(k)-Vmp_c2(k))/(const_mud_pipe1+const_mud2_pipe1+const_spacer1_pipe1+ const_spacer2_pipe1+ const_cem_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud);
        Vsp2 (k)        = Vsp_c2(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of mud
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud);
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem);% solving for new compressed volume of cement
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)/Ap)+hd(k-1)+(Vmp2(k)/Ap); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_spacer1_pipe2+const_spacer2_pipe2+const_cem_pipe2));
        Pdha(k)         = Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1)-qc(k-1));
        
        
    elseif statec(k) == 5 % Spacer enters annulus        
        % Pipe
        Vcp_c(k)        = Vcp_c(k-1);
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        Vsp_c1(k)       = max(0,Vsp1(k-1) - dt*(qb(k-1)));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer1_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp1(k)         = max(0,Vsp_c1(k)*(1-deltaP(k)/B_spacer));
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)/Ap)+hd(k-1)+(Vmp2(k)/Ap); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2+const_spacer1_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qc(k-1));
        Vsa_c(k)        = Vsa(k-1) + dt*(qb(k-1));
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k))/(const_mud_an1+const_spacer_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Lsa(k)          = Vsa(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qc(k-1))/(const_mud_an2+const_spacer_an2);
        
    elseif statec(k) == 6 % Cement enters annulus     
         % Pipe
        Vcp_c(k)        = max(0,Vcp(k-1)-dt*(qb(k-1)));
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vsp_c1(k) = 0;
        Vsp1(k) = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lsp1(k)          =0;
        Lsp2(k)         = Vsp2(k)/Ap;
        
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qc(k-1));
        Vsa_c(k) = Vsa_c(k-1);
        Vca_c(k)        = Vca(k-1) + dt*(qb(k-1));
        
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        const_cem_an1 = Vca_c(k)/B_cem;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k)-Vca_c(k))/(const_mud_an1+const_spacer_an1+const_cem_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vca(k) = Vca_c(k)*(1-deltaP(k)/B_cem);
        Lsa(k)          = Vsa(k)/Aa;
        Lca(k) = Vca(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        const_cem_an2= Vca(k)/B_cem;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qc(k-1))/(const_mud_an2+const_spacer_an2+const_cem_an2);
        
        
    else %done
       
        qp(k)           = max(0, qp(k-1) - dt*(qp_mud/tramp));
    
        % Pipe
        Vcp_c(k)        = max(0,Vcp(k-1)-dt*(qb(k-1)));
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vsp_c1(k) = 0;
        Vsp1(k) = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lsp1(k)          =0;
        Lsp2(k)         = Vsp2(k)/Ap;
        
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qc(k-1));
        Vsa_c(k) = Vsa_c(k-1);
        Vca_c(k)        = Vca(k-1) + dt*(qb(k-1));
        
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        const_cem_an1 = Vca_c(k)/B_cem;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k)-Vca_c(k))/(const_mud_an1+const_spacer_an1+const_cem_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vca(k) = Vca_c(k)*(1-deltaP(k)/B_cem);
        Lsa(k)          = Vsa(k)/Aa;
        Lca(k) = Vca(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        const_cem_an2= Vca(k)/B_cem;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qc(k-1))/(const_mud_an2+const_spacer_an2+const_cem_an2);  
    end    
  
    if hd(k-1) ~= 0
        Pp(k) = 0;
    end
    
    %% Dynamics for total wellbore
    qb(k)        = max(0,qb(k-1) + dt/(Mp(k-1)+Ma(k-1))*((Pp(k-1)  - Pc(k-1) - Fp(k-1)-Fa(k-1) + Gp(k-1)-Ga(k-1))));
    qavg_pipe(k) = max(0,qavg_pipe(k-1) + dt/Mp(k-1)*(Pp(k-1) - Fp(k-1) - Pdha(k-1) + Gp(k-1)));
    qc(k)        = max(0,qc(k-1) + dt/Ma(k-1)*(Pdha(k-1) - Fa(k-1) - Pc(k-1) - Ga(k-1)));
   
    
    if  Pp(k) == 0 && ((Gp(k-1)-Ga(k-1)-Fa(k-1)-Fp(k-1)-Pc(k-1))) ~= 0
        hd(k) = max(0,hd(k-1) + (dt/Ap)*(qb(k-1)-qp(k-1)));
    else
        hd(k) = 0;
    end
        
      
    %% PI CONTROLLER    
    ref(k)  = r0;
    eI(k)   = eI(k-1) + dt*e(k-1);
    e(k)    = Pdha(k) - ref(k) + noise(k);     
    u(k)    = Kp*e(k) + Kp/Ti*eI(k);
    
    if u(k) < umin
        u(k)  = umin;           
        eI(k) = Ti/Kp*(u(k) - Kp*e(k));        
    elseif u(k) > 1
        u(k)  = 1;
        eI(k) = Ti/Kp*(u(k) - Kp*e(k));  
    end
    
    if u(k)  == umin
        qc(k) = 0;
    else
        Pc(k) = rho_mud*(qc(k)/Cv/u(k))^2;   
    end
    %% Update variables    
  
    Lsp(k) = Lsp1(k) + Lsp2(k);
    Lma(k) = L_SI - Lca(k) - Lsa(k);
    Lmp(k) = L_SI - Lcp(k) - hd(k)-Lsp(k);
    
    q_avgp =  qavg_pipe(k); 
    q_avga =  qc(k); 
    
    Mp(k)  = (rho_mud*Lmp(k)/(Ap) + rho_cem*Lcp(k)/(Ap) + rho_spacer*Lsp(k)/(Ap));
    Ma(k)  = (rho_mud*Lma(k)/(Aa) + rho_cem*Lca(k)/(Aa) + rho_spacer*Lsa(k)/(Aa));
    
    Gp(k)  = rho_mud*g*Lmp(k) + rho_cem*g*Lcp(k) + rho_spacer*g*Lsp(k);
    Ga(k)  = rho_mud*g*Lma(k) + rho_cem*g*Lca(k) + rho_spacer*g*Lsa(k);
    
    Fp(k)  = unified_friction_pipe(Sd_i,q_avgp,Lmp(k),PV_mud,YP_SI_mud,tau_y_mud,rho_mud) + unified_friction_pipe(Sd_i,q_avgp,Lcp(k),PV_cem,YP_SI_cem,tau_y_cem,rho_cem) + unified_friction_pipe(Sd_i,q_avgp,Lsp(k),PV_spacer,YP_SI_spacer,tau_y_spacer,rho_spacer);   
    Fa(k)  = unified_friction_ann(Cd_o,Od,q_avga,Lma(k),PV_mud,YP_SI_mud,tau_y_mud,rho_mud) + unified_friction_ann(Cd_o,Od,q_avga,Lca(k),PV_cem,YP_SI_cem,tau_y_cem,rho_cem) + unified_friction_ann(Cd_o,Od,q_avga,Lsa(k),PV_spacer,YP_SI_spacer,tau_y_spacer,rho_spacer);   

    if qc(k)==0
       Pc(k) = max(0,Pdha(k)-Fa(k)-Ga(k));    
    end    
        if qp(k) == 0
            Pp(k) = 0;
        end
end
% Plots