
%% Define parameters
close all;clc;clear all;
L_SI        = 4000;                               % length of well, [m]
rho_mud     = 1300;                               % mud density bentonite slurry, [kg/m^3]
rho_cem     = 2000;                               % Portland cement, [kg/m^3]
rho_spacer  = 1600;                               % spacer density, [kg/m^3]
Cd_o        = 13.375;                             % casing outer diameter, [in]
Sd_i        = 5;                                  % stinger inner diameter, [in]
Od          = 17.5;                               % Openhole diameter, [in]
PV_mud      = 30;                                 % plastic viscosity mud, [cP]
PV_spacer   = 62;                                 % plastic viscosity spacer, [cP]
PV_cem      = 334.2;                              % plastic viscosity cement, [cP]
YP_SI_mud   = 13.5;                               % yield point mud, [Pa]
YP_SI_spacer= 6.3;                                % yield point spacer, [Pa]
YP_SI_cem   = 27;                                 % yield point cement, [Pa]
tau_y_mud   = 0.75;                               % yield stress mud, [lb/100ft^2]
tau_y_spacer= 0.75;                               % yield stress spacer, [lb/100ft^2] 
tau_y_cem   = 0.75;                               % yield stress cement, [lb/100ft^2]
g           = 9.81;                               % gravity constant, [m/s^2]
in2m        = 0.0254;                             % multiplier from in to m
lpm2m3s     = 1/60000;                            % multiplier from lpm to m3/s
bar2Pa      = 1e5;                                % multiplier from bar to Pa
B_mud       = 1.5*10^4 * bar2Pa;                  % isothermal bulk modulus mud [Pa]
B_spacer    = 1.5*10^4*bar2Pa;                    % isothermal bulk modulus spacer [Pa]
B_cem       = 40*10^4 * bar2Pa;                   % isothermal bulk modulus cement [Pa]
Ap          = pi*(Sd_i*in2m/2)^2;                 % Cross sectional area of stinger [m^2]
Aa          = pi*((Od*in2m/2)^2-(Cd_o*in2m/2)^2); % Cross sectional area of annulus [m^2]
V_tp        = Ap*L_SI;                            % total volume pipe [m^3]
V_ta        = Aa*L_SI;                            % total volume annulus [m^3]
Vst1        = 10;                                 % pre-flush-spacer [m^3]
Vst2        = 1;                                  % post-flush-spacer [m^3]

t_m = 500;                                        % time pumping mud before cement 
dt  = 0.01;                                       % time step [s]
T   = 7200;                                       % total duration of operation [s]
t   = dt:dt:T;
N   = length(t);
qp_mud = 1000*lpm2m3s;                            % displacing mud rate [m^3/s]
qp_cem = 500*lpm2m3s;                             % pumping cement rate [m^3/s]
qp_mud_dis = 1500*lpm2m3s;                        % displacement mud rate [m^3/s] 
P0  = 0e5;                                        % gauge Pressure pressure = 0 bar
Lmp0= L_SI;                                       % initial length of mud in stinger [m]
Lma0= Lmp0;                                       % initial length of mud in annulus [m]
Lcem_an = 300;                                    % length of cement in annulus [m]
Vct = Lcem_an*Aa;                                 % total volume of cement to be pumped [m3]

% Pre-allocation a=annulus, p=pipe, c=cement, s=spacer, m=mud, f=front

qp        = zeros(1,N);
Ga        = zeros(1,N);
Gp        = zeros(1,N);
Mp        = zeros(1,N);
Ma        = zeros(1,N);
qb        = zeros(1,N);
qout      = zeros(1,N);  
qavg_pipe = zeros(1,N);
Fp        = zeros(1,N); 
Fa        = zeros(1,N); 
Pp        = zeros(1,N); 
Pdhp      = zeros(1,N);
Pdha      = zeros(1,N);
deltaP    = zeros(1,N);
Lcp       = zeros(1,N);
Lcpf      = zeros(1,N);
Lca       = zeros(1,N);
Lma       = zeros(1,N);
Lmp       = zeros(1,N);
Lsp       = zeros(1,N);
Lspf      = zeros(1,N);
Lsp1      = zeros(1,N);
Lsp2      = zeros(1,N);
Lsa       = zeros(1,N);
hd        = zeros(1,N);
Vcp       = zeros(1,N);
Vcp_c     = zeros(1,N);
Vmp1      = zeros(1,N);
Vmp2      = zeros(1,N);
Vmp_c1    = zeros(1,N);
Vmp_c2    = zeros(1,N);
Vma_c     = zeros(1,N);
Vca_c     = zeros(1,N);
Vca       = zeros(1,N);
Vma       = zeros(1,N);
Vsp1      = zeros(1,N);
Vsp_c1    = zeros(1,N);
Vsp2      = zeros(1,N);
Vsp_c2    = zeros(1,N);
Vsa       = zeros(1,N);
Vsa_c     = zeros(1,N);
statec    = zeros(1,N);


%% Initial conditions

qp(1)        = qp_mud;  
qb(1)        = qp_mud;  
qout(1)      = qp_mud; 
qavg_pipe(1) = qp_mud;
Ga(1)        = rho_mud*g*L_SI;
Gp(1)        = rho_mud*g*L_SI;
Mp(1)        = rho_mud*L_SI/((pi*(Sd_i^2)/4)*in2m^2);
Ma(1)        = rho_mud*L_SI/((pi*((Od^2)-(Cd_o^2))/4)*in2m^2);

Fp(1)        = unified_friction_pipe(Sd_i,qp_mud,L_SI,PV_mud,YP_SI_mud,tau_y_mud,rho_mud); 
Fa(1)        = unified_friction_ann(Cd_o,Od,qp_mud,L_SI,PV_mud,YP_SI_mud,tau_y_mud,rho_mud);
Pdhp(1)      = P0 + Fa(1) + Ga(1);
Pdha(1)      = P0 + Fa(1) + Ga(1);
Pp(1)        = Pdha(1) + Fp(1) - Gp(1);
r0           = Pdha(1);
Lma(1)       = Lma0;
Lmp(1)       = Lmp0;
Lmp1(1)      = Lmp0;
Vmp1(1)      = V_tp;
Vma(1)       = V_ta;



%% Flow
alpha_pipe = 0.1;
alpha_ann  = 0.1;
tramp = 20;%sec
headindp  = true;
for k = 2:N
    if t(k) < 50 % bygger opp pumpetrykk
        qp(k) = qp_mud; %qp0;
    elseif (t(k) < 100)
        qp(k) = min(qp_mud*1.2 , qp(k-1)+dt*qp_mud/tramp); %min(qp0*2,qp(k-1)+dt*qp0/tramp);
    elseif t(k) < 150
        qp(k) = min(qp_mud*1.5 ,  qp(k-1)+dt*qp_mud/tramp); %min(qp0,  qp(k-1)+dt*qp0/tramp);;
    elseif t(k) < 250 
        qp(k) = qp_mud*1.5;
    else 
        qp(k) = max(qp_cem, qp(k-1)-dt*qp_mud/tramp); %min(qp0*2,qp(k-1)+dt*qp0/tramp);
    end
    %State machine
    if statec(k-1) == 0
        if t(k) <= t_m % pumping mud
            statec(k) = 0;
        else
            statec(k) = 1;
        end
    elseif statec(k-1) == 1 
        if Vsp1(k-1) < Vst1 %pumping spacer
            statec(k) = 1;
        else 
            statec(k) = 2;
        end
    elseif statec(k-1) == 2
        if Vcp(k-1) < Vct %pumping cement
            statec(k) = 2;
        else
            statec(k) = 3;
        end
    elseif statec(k-1) == 3 
        if Vsp2(k-1) < Vst2 %pumping spacer
            statec(k) = 3;
        else 
            statec(k) = 4;
        end
    elseif statec(k-1) == 4 %pumping mud
        if Lspf(k-1) < L_SI % 
            statec(k) = 4;
        else
            statec(k) = 5;
        end
    elseif statec(k-1) == 5
      if Lcpf(k-1) < L_SI % 
            statec(k) = 5;
      else
          statec(k) = 6;
      end
        elseif statec(k-1) == 6 
        if  Vcp(k-1) > 0.449 % 1.752 naar qcem = 1000 lpm, naar qcem = 500 lpm --> 0.449
            statec(k) = 6;
        else
            statec(k) = 7;
            kend      = N;
        end
        elseif statec(k-1) == 7
        statec(k) = 7;
    end
    
    if statec(k) >= 4
                    qp(k) = min(qp_mud_dis, qp(k-1)+dt*qp_mud/tramp);
    end
    
    if statec(k) == 0 % pumping mud  
        Vmp1(k)         = V_tp;
        Lmp1(k) = V_tp/Ap;
        Lmp2(k) = 0;
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Vmp_c1(k)       = V_tp;
        Vcp(k)          = 0; 
        Lcp(k)          = Vcp(k)/Ap;
        Lcpf(k)         = Lcp(k); 
        
        Pp(k)           = max(0,Pp(k-1) + dt*(B_mud/V_tp)*(qp(k-1) - qb(k-1)));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qout(k-1)));
    elseif statec(k) == 1 % pumping spacer
        Vmp_c1(k)       = Vmp1(k-1) - dt*(qb(k-1));
        Vsp_c1(k)       = Vsp1(k-1) + dt*(qp(k-1));
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_spacer_pipe1 = Vsp_c1(k)/B_spacer;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k))/(const_mud_pipe1+const_spacer_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsp1(k)          = Vsp_c1(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Lsp1(k)          = Vsp1(k)/Ap;
        Lmp1(k) = Vmp1(k)/Ap;
        Lspf(k)         = Lsp1(k) +hd(k-1);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_spacer_pipe2 = Vsp1(k)/B_spacer;
    
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+const_spacer_pipe2));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qout(k-1)));
    elseif statec(k) == 2 % pumping cement
        Vmp_c1(k)       = Vmp1(k-1) - dt*(qb(k-1));
        Vcp_c(k)        = Vcp(k-1) + dt*(qp(k-1));
        Vsp_c1(k)       = Vsp_c1(k-1);
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        const_spacer_pipe1=Vsp_c1(k)/B_spacer;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vcp_c(k)-Vsp_c1(k))/(const_mud_pipe1+const_cem_pipe1+const_spacer_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem); % solving for new compressed volume of cement
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Lcp(k)          = Vcp(k)/Ap;
        Lcpf(k)         = Lcp(k)+hd(k-1);
        Lsp1(k)          = Vsp1(k)/Ap;
        Lspf(k)         = Lcpf(k)+Lsp1(k);
         Lmp1(k) = Vmp1(k)/Ap;
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_cem_pipe2 = Vcp(k)/B_cem;
        const_spacer_pipe2=Vsp1(k)/B_spacer;
    
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+const_cem_pipe2+const_spacer_pipe2));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qout(k-1)));
    elseif statec(k) == 3  % Cement er ferdig pumpet, starter aa pumpe spacer 2           
        Vcp_c(k)        = Vcp_c(k-1); 
        Vsp_c1(k)       = Vsp_c1(k-1);
        Vmp_c1(k)       = max(0,Vmp1(k-1) - dt*(qb(k-1))); 
        Vsp_c2(k)       = Vsp2(k-1) + dt*(qp(k-1));
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k)-Vsp_c2(k)-Vcp_c(k))/(const_mud_pipe1+const_spacer1_pipe1+ const_spacer2_pipe1+ const_cem_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud);
        Vsp2 (k)        = Vsp_c2(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of mud
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem);% solving for new compressed volume of cement
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)+Vmp2(k))/Ap+hd(k-1); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
         Lmp1(k) = Vmp1(k)/Ap;
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_mud2_pipe2=Vmp2(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_mud2_pipe2+ const_spacer1_pipe2+const_spacer2_pipe2+const_cem_pipe2));
        Pdha(k)         = Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1)-qout(k-1));
        
    elseif statec(k) == 4 %cement ferdig pumpet, spacer ferdig pumpet, pumping mud! 
        
        Vcp_c(k)        = Vcp_c(k-1); 
        Vsp_c1(k)       = Vsp_c1(k-1);
        Vsp_c2(k)       = Vsp_c2(k-1);
        Vmp_c1(k)       = max(0,Vmp1(k-1) - dt*(qb(k-1))); 
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_mud2_pipe1=Vmp_c2(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k)-Vsp_c2(k)-Vcp_c(k)-Vmp_c2(k))/(const_mud_pipe1+const_mud2_pipe1+const_spacer1_pipe1+ const_spacer2_pipe1+ const_cem_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud);
        Vsp2 (k)        = Vsp_c2(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of mud
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud);
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem);% solving for new compressed volume of cement
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)/Ap)+hd(k-1)+(Vmp2(k)/Ap); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
         Lmp1(k) = Vmp1(k)/Ap;
          Lmp2(k) = Vmp2(k)/Ap;
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_spacer1_pipe2+const_spacer2_pipe2+const_cem_pipe2));
        Pdha(k)         = Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1)-qout(k-1));
        
        
    elseif statec(k) == 5 % Spacer enters annulus        
        % Pipe
        Vcp_c(k)        = Vcp_c(k-1);
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        Vsp_c1(k)       = max(0,Vsp1(k-1) - dt*(qb(k-1)));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer1_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp1(k)         = max(0,Vsp_c1(k)*(1-deltaP(k)/B_spacer));
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)/Ap)+hd(k-1)+(Vmp2(k)/Ap); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
           Lmp1(k) = Vmp1(k)/Ap;
          Lmp2(k) = Vmp2(k)/Ap;
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2+const_spacer1_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qout(k-1));
        Vsa_c(k)        = Vsa(k-1) + dt*(qb(k-1));
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k))/(const_mud_an1+const_spacer_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Lsa(k)          = Vsa(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qout(k-1))/(const_mud_an2+const_spacer_an2);
        
    elseif statec(k) == 6 % Cement enters annulus     
         % Pipe
        Vcp_c(k)        = max(0,Vcp(k-1)-dt*(qb(k-1)));
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vsp_c1(k) = 0;
        Vsp1(k) = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lsp1(k)          =0;
        Lsp2(k)         = Vsp2(k)/Ap;
           Lmp1(k) = Vmp1(k)/Ap;
          Lmp2(k) = Vmp2(k)/Ap;
        
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qout(k-1));
        Vsa_c(k) = Vsa_c(k-1);
        Vca_c(k)        = Vca(k-1) + dt*(qb(k-1));
        
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        const_cem_an1 = Vca_c(k)/B_cem;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k)-Vca_c(k))/(const_mud_an1+const_spacer_an1+const_cem_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vca(k) = Vca_c(k)*(1-deltaP(k)/B_cem);
        Lsa(k)          = Vsa(k)/Aa;
        Lca(k) = Vca(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        const_cem_an2= Vca(k)/B_cem;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qout(k-1))/(const_mud_an2+const_spacer_an2+const_cem_an2);
        
        
    else %done
       
        qp(k)           = max(0, qp(k-1) - dt*(qp_mud/tramp));
    
        % Pipe
        Vcp_c(k)        = max(0,Vcp(k-1)-dt*(qb(k-1)));
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vsp_c1(k) = 0;
        Vsp1(k) = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lsp1(k)          =0;
        Lsp2(k)         = Vsp2(k)/Ap;
           Lmp1(k) = Vmp1(k)/Ap;
          Lmp2(k) = Vmp2(k)/Ap;
        
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qout(k-1));
        Vsa_c(k) = Vsa_c(k-1);
        Vca_c(k)        = Vca(k-1) + dt*(qb(k-1));
        
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        const_cem_an1 = Vca_c(k)/B_cem;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k)-Vca_c(k))/(const_mud_an1+const_spacer_an1+const_cem_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vca(k) = Vca_c(k)*(1-deltaP(k)/B_cem);
        Lsa(k)          = Vsa(k)/Aa;
        Lca(k) = Vca(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        const_cem_an2= Vca(k)/B_cem;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qout(k-1))/(const_mud_an2+const_spacer_an2+const_cem_an2);  
    end    
    
    if hd(k-1) ~= 0
        Pp(k) = 0;
    end
    
    %% Dynamics for the total wellbore
    qb(k)        = max(0,qb(k-1) + dt/(Mp(k-1)+Ma(k-1))*((Pp(k-1)  - P0 - Fp(k-1)-Fa(k-1) + Gp(k-1)-Ga(k-1))));
    qavg_pipe(k) = max(0,qavg_pipe(k-1) + dt/Mp(k-1)*(Pp(k-1) - Fp(k-1) - Pdha(k-1) + Gp(k-1)));
    qout(k)      = max(0,qout(k-1) + dt/(Ma(k-1))*(Pdha(k-1) - Fa(k-1) - P0 - Ga(k-1)));
    
    if  Pp(k) == 0 && ((Gp(k-1)-Ga(k-1)-Fa(k-1)-Fp(k-1))) ~= 0
        hd(k) = max(0,hd(k-1) + (dt/Ap)*(qb(k-1)-qp(k-1)));
    else
        hd(k) = hd(k-1);
    end    
   
    %% Update variables
    Lsp(k) = Lsp1(k) + Lsp2(k);
    Lma(k) = L_SI - Lca(k) - Lsa(k);
    Lmp(k) = L_SI - Lcp(k) - hd(k)-Lsp(k);
        
    q_avgp = qavg_pipe(k);
    q_avga = qout(k);
    
    Mp(k)  = (rho_mud*Lmp(k)/(Ap) + rho_cem*Lcp(k)/(Ap) + rho_spacer*Lsp(k)/(Ap));
    Ma(k)  = (rho_mud*Lma(k)/(Aa) + rho_cem*Lca(k)/(Aa) + rho_spacer*Lsa(k)/(Aa));
    Gp(k)  = rho_mud*g*Lmp(k) + rho_cem*g*Lcp(k) + rho_spacer*g*Lsp(k);
    Ga(k)  = rho_mud*g*Lma(k) + rho_cem*g*Lca(k) + rho_spacer*g*Lsa(k);
    Fp(k)  = unified_friction_pipe(Sd_i,q_avgp,Lmp(k),PV_mud,YP_SI_mud,tau_y_mud,rho_mud) + unified_friction_pipe(Sd_i,q_avgp,Lcp(k),PV_cem,YP_SI_cem,tau_y_cem,rho_cem) + unified_friction_pipe(Sd_i,q_avgp,Lsp(k),PV_spacer,YP_SI_spacer,tau_y_spacer,rho_spacer);   
    Fa(k)  = unified_friction_ann(Cd_o,Od,q_avga,Lma(k),PV_mud,YP_SI_mud,tau_y_mud,rho_mud) + unified_friction_ann(Cd_o,Od,q_avga,Lca(k),PV_cem,YP_SI_cem,tau_y_cem,rho_cem) + unified_friction_ann(Cd_o,Od,q_avga,Lsa(k),PV_spacer,YP_SI_spacer,tau_y_spacer,rho_spacer);   
   
        if qp(k) == 0
            Pp(k) = 0;
        end
end
% Plots