
%% Define parameters
close all; clear all; clc;
L_SI        = 4000; % length of well, [m]
rho_spacer  = 1600;
L_SI_riser  = 400;
hssp        = L_SI_riser; % depth of mud pump, [m]
mrl_hoz     = 5; % horizontal length of discharge line, [m]
suc_line    = 5; % horizontal length of suction line, [m]
mrl_d       = 6; % mud return line inner diameter, [in]
rho_mud     = 1300; % mud weight bentonite slurry, [kg/m^3]
rho_cem     = 2000; % Portland cement, [kg/m^3]fd
Cd_o        = 13.375; % casing outer diameter, [in]
Sd_i        = 5; % stinger inner diameter, [in]
Od          = 17.5; % Openhole diameter, [in]
PV_spacer   = 62;
YP_SI_spacer= 6.3;
tau_y_spacer= 0.75;
PV_mud      = 30; % plastic viscosity mud, [cP]
PV_cem      = 334.2; % plastic viscosity cement, [cP]
YP_SI_mud   = 13.5; %yield point t,Pp/bar2Pa,mud, [Pa]
YP_SI_cem   = 27; %yield point cement, [Pa]
tau_y_mud   = 0.75; % yield stress mud, [lb/100ft^2]
tau_y_cem   = 0.75; % yield stress cement, [lb/100ft^2]
g           = 9.81; % gravity constant, [m/s^2]
max_hr      = hssp-5e5/(rho_mud*g); % laveste niv? i riser
in2m        = 0.0254; % multiplier from in to m
lpm2m3s     = 1/60000;% multiplier from lpm to m3/s
bar2Pa      = 1e5;% multiplier from bar to Pa
B_mud       = 1.5*10^4 * bar2Pa; % isothermal bulk modulus mud [Pa]
B_cem       = 40*10^4 * bar2Pa; % isothermal bulk modulus mud [Pa]
B_spacer    = 1.5*10^4*bar2Pa;
Ap          = pi*(Sd_i*in2m/2)^2;%m2
Aa          = pi*((Od*in2m/2)^2-(Cd_o*in2m/2)^2);
Amrl        = pi*(mrl_d*in2m/2)^2;%m2
V_tp        = Ap*L_SI;%m3 total volume pipe
V_ta        = Aa*L_SI;%m3 total volume annulus
Vst1        = 10;%m3 % 60 bbl pre-spacer
Vst2        = 1;%m3 % 5 bbl flush-spacer
G0          = rho_mud*g*L_SI;%static pressure downhole full riser (no flow)
offset      = 0;
r0          = G0-offset;

% Subsea pump data
np          = 2; %number of pumps
c0          = 4.17*1e-5;% pump constant
c1          = 6.83*1e-2;% pump constant
c2          = 115;     % pump constant

wssp_max    = 2000; % rpm max pump speed
wssp_min    = 600;  % rpm min pump speed

t_m        = 500; %s time pumping mud before cement 
dt         = 0.01;%s
T          = 7200;%s
t          = dt:dt:T;
N          = length(t);
qp_mud     = 1000*lpm2m3s;%m3/s
qp_cem     = 500*lpm2m3s; %m3/s
qp_mud_dis = 1500*lpm2m3s;%m3/s
P0         = 0e5; % Gauge Pressure Atmospheric pressure = 0 bar
Lcem       = 300;
Vct        = Lcem*Aa; %total cement to be pumped [m3]

% Allocate time series
qp       = zeros(1,N);
qb       = zeros(1,N);
qssp     = zeros(1,N); 
qriser   = zeros(1,N);
qavg_pipe= zeros(1,N);
qavg_ann = zeros(1,N);
Ga       = zeros(1,N);
Ga2      = zeros(1,N);
Gp       = zeros(1,N);
Grl      = zeros(1,N);
Mp       = zeros(1,N);
Ma       = zeros(1,N);
Mrl      = zeros(1,N);
Fp       = zeros(1,N); 
Fa       = zeros(1,N); 
Frl      = zeros(1,N);
Pp       = zeros(1,N); 
Pdha     = zeros(1,N);
DPspm    = zeros(1,N); % delta pressure created by pump
Pspm_in  = zeros(1,N); % inlet pressure pump
Pspm_out = zeros(1,N); % outlet pressure pump
deltaP   = zeros(1,N);
Lcp      = zeros(1,N);
Lcpf     = zeros(1,N);
Lca      = zeros(1,N);
Lma_bp   = zeros(1,N); % Length mud below pump(hssp) in annulus
Lma_tot  = zeros(1,N);
Lmp      = zeros(1,N);
Lsa      = zeros(1,N);
Lsp      = zeros(1,N);
Lspf     = zeros(1,N);
Lsp1     = zeros(1,N);
Lsp2     = zeros(1,N);
hr       = zeros(1,N);
hd       = zeros(1,N);
Vcp      = zeros(1,N);%cement in pipe
Vcp_c    = zeros(1,N);%cement in pipe before compressed
Vmp1     = zeros(1,N);%mud ahead of cement
Vmp2     = zeros(1,N);%displaced mud behind cement
Vmp_c1   = zeros(1,N);%displaced mud in pipe before compressed
Vmp_c2   = zeros(1,N);%displacing mud in pipe before compressed
Vma_c    = zeros(1,N);%mud in annulus before compressed
Vca_c    = zeros(1,N);%cement in annulus before compressed
Vca      = zeros(1,N);%cement in annulus
Vma      = zeros(1,N);%mud in annulus
Vap      = zeros(1,N);%air gap in annulus
Vsp1     = zeros(1,N);%cement in pipe
Vsp_c1   = zeros(1,N);%cement in pipe before compressed
Vsp2     = zeros(1,N);%cement in pipe
Vsp_c2   = zeros(1,N);%cement in pipe before compressed
Vsa      = zeros(1,N);
Vsa_c    = zeros(1,N);

wssp_ref = zeros(1,N);
wssp     = zeros(1,N);
e        = zeros(1,N);
eI       = zeros(1,N);
u        = zeros(1,N);
ref      = zeros(1,N);%Desired downhole pressure
statec   = zeros(1,N);%0=pump mud, 1=pump pre-spacer, 2=pump cement, 3=pump flush-spacer, 4=pump mud
sol      = zeros(1,N);
H        = zeros(1,N);%Pump head [m]

%Controller
Kp          = 0.7e-7;%0.7*1e-8;% 1 bar offset will give 0.7% change in choke pos
Ti          = 20;%sec
umin        = 0;
umax        = 1;
tau_ssp     = 5; % pump tramp time, 5 seconds [s]

%% Initial conditions
qp(1)       = qp_mud; 
qb(1)       = qp_mud; 
qssp(1)     = qp_mud;
qriser(1)   = qp_mud;
qavg_pipe(1)= qp_mud;
qavg_ann(1)=qp_mud;

Fp(1)       = unified_friction_pipe(Sd_i,qp_mud,L_SI,PV_mud,YP_SI_mud,tau_y_mud,rho_mud); % friction pressure drop pipe, [Pa] 
Fa(1)       = unified_friction_ann(Cd_o,Od,qp_mud,L_SI-hssp,PV_mud,YP_SI_mud,tau_y_mud,rho_mud); % friction pressure drop annulus, [Pa]
Frl(1)      = unified_friction_pipe(mrl_d,qp_mud,hssp+mrl_hoz,PV_mud,YP_SI_mud,tau_y_mud,rho_mud); % friction pressure mud return line, [Pa] 

%Pdha0  = rho_mud*g*L_SI + Fa(1); % Downhole pressure full pipe and flow%
diff_p    = G0 + Fa(1) - r0;%Pdha0 - r0; % pressure difference between flow and no flow

hr0       = diff_p/(rho_mud*g);
hr(1)     = hr0;
Lma_bp(1) = L_SI - hssp;
Lma_tot(1)= L_SI - hr0;
Lmp(1)    = L_SI;
Vm1p_c(1) = V_tp;

Ga(1)       = rho_mud*g*(L_SI - hr0);
Ga2(1)      = rho_mud*g*(L_SI - hssp);
Gp(1)       = rho_mud*g*L_SI;
Grl(1)      = rho_mud*g*hssp;
Mp(1)       = rho_mud*L_SI/Ap;%kg/m4
Ma(1)       = rho_mud*(L_SI-hr0)/Aa;%kg/m4
Mrl(1)      = rho_mud*(hssp)/Amrl;

Pdha(1)     = Ga(1) + Fa(1);
Pspm_in(1)  = (hssp-hr0)*rho_mud*g;
Pspm_out(1) = Grl(1) + Frl(1);
DPspm(1)    = Pspm_out(1) - Pspm_in(1);
H(1)        = DPspm(1)/(np*rho_mud*g);
Pp(1)       = Pdha(1) + Fp(1) - Gp(1);

wssp0       = (c1*qp_mud + sqrt((c1*qp_mud)^2+4*c0*(c2*qp_mud^2+H(1))))/(2*c0);
wssp(1)     = wssp0;
wssp_ref(1) = wssp0;
u0          = umin+(wssp0-wssp_min)/(wssp_max-wssp_min)*(umax-umin);
u(1)        = u0;
ref(1)      = r0;
e(1)        = Pdha(1) - ref(1);
eI(1)       = Ti/Kp*(u(1) - Kp*e(1));        

% hdot = 2*qp0/Ap;
% Pdot = g*(rho_cem-rho_mud)*2*qp0/Ap;
% qadd = Pdot*V_tp/B_mud;
% fprintf(1,'Cement front velocity %.2f m/s, pump pressure drops with %.1f bar/min and additional bit flow %.0f lpm\n', ...
%     hdot, Pdot*1e-5*60, qadd*60000);


%% Flow
alpha_pipe= 0;
alpha_ann = 0.5;
tramp     = 20;%sec
noise     = zeros(1,N); %0.02e5*randn(1,N);
headindp  = true;
kend      = N;%k-2;

for k = 2:N
      if t(k) < 50 % bygger opp pumpetrykk
        qp(k) = qp_mud; %qp0;
    elseif (t(k) < 100)
        qp(k) = min(qp_mud*1.2 , qp(k-1)+dt*qp_mud/tramp); %min(qp0*2,qp(k-1)+dt*qp0/tramp);
    elseif t(k) < 150
        qp(k) = min(qp_mud*1.5 ,  qp(k-1)+dt*qp_mud/tramp); %min(qp0,  qp(k-1)+dt*qp0/tramp);;
 elseif t(k) < 250 
        qp(k) = qp_mud*1.5;
    else  
        qp(k) = max(qp_cem, qp(k-1)-dt*qp_mud/tramp); %min(qp0*2,qp(k-1)+dt*qp0/tramp);
      end
    
    %State machine
    if statec(k-1) == 0
        if t(k) <= t_m % pumping mud
            statec(k) = 0;
        else
            statec(k) = 1;
        end
    elseif statec(k-1) == 1 
        if Vsp1(k-1) < Vst1 %pumping spacer
            statec(k) = 1;
        else 
            statec(k) = 2;
        end
    elseif statec(k-1) == 2
        if Vcp(k-1) < Vct %pumping cement
            statec(k) = 2;
        else
            statec(k) = 3;
        end
    elseif statec(k-1) == 3 
        if Vsp2(k-1) < Vst2 %pumping spacer
            statec(k) = 3;
        else 
            statec(k) = 4;
        end
    elseif statec(k-1) == 4 %pumping mud
        if Lspf(k-1) < L_SI % 
            statec(k) = 4;
        else
            statec(k) = 5;
        end
    elseif statec(k-1) == 5
      if Lcpf(k-1) < L_SI % Cement front reach bottom of well
            statec(k) = 5;
      else
          statec(k) = 6;
      end
        elseif statec(k-1) == 6 
        if  Vcp(k-1) > 0.46 % 1.75 n?r qcem = 1000 lpm, n?r qcem = 500 lpm --> 0.46
            statec(k) = 6;
        else
            statec(k) = 7;
            kend      = N;
        end
        elseif statec(k-1) == 7
        statec(k) = 7;
    end
    
    if statec(k) >= 4
                    qp(k) = min(qp_mud_dis, qp(k-1)+dt*qp_mud/tramp);
    end
    
    if statec(k) == 0 % pumping mud  
        Vmp1(k)         = V_tp-Vap(k-1);
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Vmp_c1(k)       = V_tp-Vap(k-1);
        Vcp(k)          = 0; 
        Lcp(k)          = Vcp(k)/Ap;
        Lcpf(k)         = Lcp(k); 
        
        Pp(k)           = max(0,Pp(k-1) + dt*(B_mud/V_tp)*(qp(k-1) - qb(k-1)));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qriser(k-1)));
    elseif statec(k) == 1 % pumping spacer
        Vmp_c1(k)       = Vmp1(k-1) - dt*(qb(k-1));
        Vsp_c1(k)       = Vsp1(k-1) + dt*(qp(k-1));
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_spacer_pipe1 = Vsp_c1(k)/B_spacer;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k))/(const_mud_pipe1+const_spacer_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsp1(k)          = Vsp_c1(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Lsp1(k)          = Vsp1(k)/Ap;
        Lspf(k)         = Lsp(k) +hd(k-1);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_spacer_pipe2 = Vsp1(k)/B_spacer;
    
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+const_spacer_pipe2));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qriser(k-1)));
    elseif statec(k) == 2 % pumping cement
        Vmp_c1(k)       = Vmp1(k-1) - dt*(qb(k-1));
        Vcp_c(k)        = Vcp(k-1) + dt*(qp(k-1));
        Vsp_c1(k)       = Vsp_c1(k-1);
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        const_spacer_pipe1=Vsp_c1(k)/B_spacer;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vcp_c(k)-Vsp_c1(k))/(const_mud_pipe1+const_cem_pipe1+const_spacer_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem); % solving for new compressed volume of cement
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        Lcp(k)          = Vcp(k)/Ap;
        Lcpf(k)         = Lcp(k)+hd(k-1);
        Lsp1(k)          = Vsp1(k)/Ap;
        Lspf(k)         = Lcpf(k)+Lsp1(k);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_cem_pipe2 = Vcp(k)/B_cem;
        const_spacer_pipe2=Vsp1(k)/B_spacer;
    
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+const_cem_pipe2+const_spacer_pipe2));
        Pdha(k)         = max(0,Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1) - qriser(k-1)));
    elseif statec(k) == 3  % Cement er ferdig pumpet, starter aa pumpe spacer 2           
        Vcp_c(k)        = Vcp_c(k-1); 
        Vsp_c1(k)       = Vsp_c1(k-1);
        Vmp_c1(k)       = max(0,Vmp1(k-1) - dt*(qb(k-1))); 
        Vsp_c2(k)       = Vsp2(k-1) + dt*(qp(k-1));
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k)-Vsp_c2(k)-Vcp_c(k))/(const_mud_pipe1+const_spacer1_pipe1+ const_spacer2_pipe1+ const_cem_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud);
        Vsp2 (k)        = Vsp_c2(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of mud
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem);% solving for new compressed volume of cement
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)+Vmp2(k))/Ap+hd(k-1); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_spacer1_pipe2+const_spacer2_pipe2+const_cem_pipe2));
        Pdha(k)         = Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1)-qriser(k-1));
        
    elseif statec(k) == 4 %cement ferdig pumpet, spacer ferdig pumpet, pumping mud! 
        
        Vcp_c(k)        = Vcp_c(k-1); 
        Vsp_c1(k)       = Vsp_c1(k-1);
        Vsp_c2(k)       = Vsp_c2(k-1);
        Vmp_c1(k)       = max(0,Vmp1(k-1) - dt*(qb(k-1))); 
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        Vma(k)          = V_ta;
        Vma_c(k)        = V_ta;
        const_mud_pipe1 = Vmp_c1(k)/B_mud;
        const_mud2_pipe1=Vmp_c2(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c1(k)-Vsp_c1(k)-Vsp_c2(k)-Vcp_c(k)-Vmp_c2(k))/(const_mud_pipe1+const_mud2_pipe1+const_spacer1_pipe1+ const_spacer2_pipe1+ const_cem_pipe1);
        Vmp1(k)         = Vmp_c1(k)*(1-deltaP(k)/B_mud);
        Vsp2 (k)        = Vsp_c2(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of mud
        Vsp1(k)         = Vsp_c1(k)*(1-deltaP(k)/B_spacer);
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud);
        Vcp(k)          = Vcp_c(k)*(1-deltaP(k)/B_cem);% solving for new compressed volume of cement
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)/Ap)+hd(k-1)+(Vmp2(k)/Ap); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
        const_mud_pipe2 = Vmp1(k)/B_mud;
        const_mud2_pipe2= Vmp2(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+const_mud2_pipe2+ const_spacer1_pipe2+const_spacer2_pipe2+const_cem_pipe2));
        Pdha(k)         = Pdha(k-1) + dt*(B_mud/V_ta)*(qb(k-1)-qriser(k-1));
        
        
    elseif statec(k) == 5 % Spacer enters annulus        
        % Pipe
        Vcp_c(k)        = Vcp_c(k-1);
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        Vsp_c1(k)       = max(0,Vsp1(k-1) - dt*(qb(k-1)));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer1_pipe1=Vsp_c1(k)/B_spacer;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer1_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp1(k)         = max(0,Vsp_c1(k)*(1-deltaP(k)/B_spacer));
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lcpf(k)         = (Vcp(k)/Ap)+(Vsp2(k)/Ap)+hd(k-1)+(Vmp2(k)/Ap); %cement front
        Lsp1(k)          = Vsp1(k)/Ap;
        Lsp2(k)         = Vsp2(k)/Ap;
        Lspf(k)         = Lcpf(k) + Lsp1(k);
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer1_pipe2= Vsp1(k)/B_spacer;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2+const_spacer1_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qriser(k-1));
        Vsa_c(k)        = Vsa(k-1) + dt*(qb(k-1));
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k))/(const_mud_an1+const_spacer_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Lsa(k)          = Vsa(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qriser(k-1))/(const_mud_an2+const_spacer_an2);
        
    elseif statec(k) == 6 % Cement enters annulus     
         % Pipe
        Vcp_c(k)        = max(0,Vcp(k-1)-dt*(qb(k-1)));
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vsp_c1(k) = 0;
        Vsp1(k) = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lsp1(k)          =0;
        Lsp2(k)         = Vsp2(k)/Ap;
        
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qriser(k-1));
        Vsa_c(k) = Vsa_c(k-1);
        Vca_c(k)        = Vca(k-1) + dt*(qb(k-1));
        
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        const_cem_an1 = Vca_c(k)/B_cem;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k)-Vca_c(k))/(const_mud_an1+const_spacer_an1+const_cem_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vca(k) = Vca_c(k)*(1-deltaP(k)/B_cem);
        Lsa(k)          = Vsa(k)/Aa;
        Lca(k) = Vca(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        const_cem_an2= Vca(k)/B_cem;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qriser(k-1))/(const_mud_an2+const_spacer_an2+const_cem_an2);
        
        
    else %done
       
        qp(k)           = max(0, qp(k-1) - dt*(qp_mud/tramp));
    
        % Pipe
        Vcp_c(k)        = max(0,Vcp(k-1)-dt*(qb(k-1)));
        Vsp_c2(k) = Vsp_c2(k-1);
        Vmp_c2(k)       = Vmp2(k-1) + dt*(qp(k-1));
        
        const_mud_pipe1 = Vmp_c2(k)/B_mud;
        const_spacer2_pipe1= Vsp_c2(k)/B_spacer;
        const_cem_pipe1 = Vcp_c(k)/B_cem;
        
        deltaP(k)       = -(V_tp-hd(k-1)*Ap-Vmp_c2(k)-Vcp_c(k)-Vsp_c1(k)-Vsp_c2(k))/( const_mud_pipe1+const_spacer2_pipe1+const_cem_pipe1);
        Vmp1(k)         = 0;
        Vmp_c1(k)       = 0;
        Vsp_c1(k) = 0;
        Vsp1(k) = 0;
        Vmp2(k)         = Vmp_c2(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vcp(k)          = max(0,Vcp_c(k)*(1-deltaP(k)/B_cem)); % solving for new compressed volume of cement
        Vsp2(k)         = max(0,Vsp_c2(k)*(1-deltaP(k)/B_spacer));
        
        Lcp(k)          = (Vcp(k)/Ap);% Cement height
        Lsp1(k)          =0;
        Lsp2(k)         = Vsp2(k)/Ap;
        
        const_mud_pipe2 = Vmp2(k)/B_mud;
        const_spacer2_pipe2= Vsp2(k)/B_spacer;
        const_cem_pipe2 = Vcp(k)/B_cem;
        
        Pp(k)           = max(0,Pp(k-1) + dt*(qp(k-1)-qb(k-1))/(const_mud_pipe2+ const_cem_pipe2+const_spacer2_pipe2));
        
        % Annulus
        Vma_c(k)        = Vma(k-1) - dt*(qriser(k-1));
        Vsa_c(k) = Vsa_c(k-1);
        Vca_c(k)        = Vca(k-1) + dt*(qb(k-1));
        
        const_spacer_an1   = Vsa_c(k)/B_spacer;
        const_mud_an1   = Vma_c(k)/B_mud;
        const_cem_an1 = Vca_c(k)/B_cem;
        deltaP(k)       = -(V_ta-Vma_c(k)-Vsa_c(k)-Vca_c(k))/(const_mud_an1+const_spacer_an1+const_cem_an1);
        Vma(k)          = Vma_c(k)*(1-deltaP(k)/B_mud); % solving for new compressed volume of mud
        Vsa(k)          = Vsa_c(k)*(1-deltaP(k)/B_spacer); % solving for new compressed volume of cement
        Vca(k) = Vca_c(k)*(1-deltaP(k)/B_cem);
        Lsa(k)          = Vsa(k)/Aa;
        Lca(k) = Vca(k)/Aa;
        const_mud_an2   = Vma(k)/B_mud;
        const_spacer_an2   = Vsa(k)/B_spacer;
        const_cem_an2= Vca(k)/B_cem;
        Pdha(k)         = Pdha(k-1) + dt*(qb(k-1)-qriser(k-1))/(const_mud_an2+const_spacer_an2+const_cem_an2);  
    end    
  
    if hd(k-1) ~= 0
        Pp(k) = 0;
    end
    
    %% PI CONTROLLER
    ref(k)  = r0;
    e(k)    = Pdha(k-1) - ref(k) + noise(k);  
    eI(k)   = eI(k-1) + dt*e(k);   
    u(k)    = Kp*e(k) + Kp/Ti*eI(k);    
    if u(k) < umin
       u(k)  = umin;           
       eI(k) = Ti/Kp*(u(k) - Kp*e(k));        
    elseif u(k) > umax 
       u(k)  = umax;
       eI(k) = Ti/Kp*(u(k) - Kp*e(k));  
    end        

    
    %Pump speed
    wssp_ref(k) = wssp_min+(u(k)-umin)*(wssp_max-wssp_min)/(umax-umin); 
    wssp(k)     = wssp(k-1) + dt/tau_ssp*(wssp_ref(k)-wssp(k-1));
    
    %% Dynamics for total wellbore
    
    qb(k)       = max(0,qb(k-1)     + dt/(Mp(k-1)+Ma(k-1))*(Pp(k-1)  -Pspm_in(k-1)- Fp(k-1)-Fa(k-1) + Gp(k-1)-Ga2(k-1)));
    qriser(k)   = max(0,qriser(k-1) + dt/Ma(k-1)*(Pdha(k-1) -Pspm_in(k-1)- Fa(k-1)-Ga2(k-1)));
    qssp(k)     = max(0,qssp(k-1) + dt/Mrl(k-1)*(Pspm_in(k-1) + np*rho_mud*g*(c0*wssp(k)^2-c1*wssp(k)*qssp(k-1)-c2*qssp(k-1)^2) - Pspm_out(k-1)));
    qavg_pipe(k)= max(0,qavg_pipe(k-1)     + dt/Mp(k-1)*(Pp(k-1)   - Pdha(k-1)    - Fp(k-1) + Gp(k-1))); 

        if qp(k) == 0   
           qssp(k) = 0;
           qb(k) = 0;
          
          
        end

    hr(k)      = min(max_hr,hr(k-1) + dt/Aa*(qssp(k-1)- qriser(k-1)));
    %U-tubing drillpipe
    if  Pp(k) == 0 && ((Gp(k-1)-Ga(k-1)-Fa(k-1)-Fp(k-1))) ~= 0
        hd(k) = max(0,hd(k-1) + (dt/Ap)*(qb(k-1)-qp(k-1)));
    else
        hd(k) = hd(k-1);
    end
    
    %% Update variables
    Pspm_in(k) = rho_mud*(hssp-hr(k))*g;    
    Vap(k) = hd(k)*Ap;    
    Lsp(k) = Lsp1(k) + Lsp2(k);
    Lma_bp(k)  = L_SI - Lca(k) - hssp-Lsa(k);
    Lma_tot(k) = Lma_bp(k) + (hssp-hr(k));
    Lmp(k)     = L_SI - Lcp(k)-hd(k)-Lsp(k);
    
        
    q_avgp     = qavg_pipe(k);
    q_avga     = (qb(k)+qriser(k))/2; % Average flow in the annulus
    q_avgm     = qssp(k);
    
    
    Mp(k)  = (rho_mud*Lmp(k)/(Ap) + rho_cem*Lcp(k)/(Ap) + rho_spacer*Lsp(k)/(Ap));
    Ma(k)  = (rho_mud*Lma_bp(k)/(Aa) + rho_cem*Lca(k)/(Aa) + rho_spacer*Lsa(k)/(Aa));
    Mrl(k) = (rho_mud*(hssp+mrl_hoz)/Amrl);
    
    Gp(k)  = rho_mud*g*Lmp(k) + rho_cem*g*Lcp(k) + rho_spacer*g*Lsp(k);
    Ga(k)  = rho_mud*g*Lma_tot(k) + rho_cem*g*Lca(k) + rho_spacer*g*Lsa(k);
    Ga2(k)     = rho_mud*g*(Lma_bp(k)) + rho_cem*g*Lca(k)+rho_spacer*g*Lsa(k);
    Grl(k)     = rho_mud*g*hssp;

    Fp(k)  = unified_friction_pipe(Sd_i,q_avgp,Lmp(k),PV_mud,YP_SI_mud,tau_y_mud,rho_mud) + unified_friction_pipe(Sd_i,q_avgp,Lcp(k),PV_cem,YP_SI_cem,tau_y_cem,rho_cem) + unified_friction_pipe(Sd_i,q_avgp,Lsp(k),PV_spacer,YP_SI_spacer,tau_y_spacer,rho_spacer);   
    Fa(k)  = unified_friction_ann(Cd_o,Od,q_avga,Lma_bp(k),PV_mud,YP_SI_mud,tau_y_mud,rho_mud) + unified_friction_ann(Cd_o,Od,q_avga,Lca(k),PV_cem,YP_SI_cem,tau_y_cem,rho_cem) + unified_friction_ann(Cd_o,Od,q_avga,Lsa(k),PV_spacer,YP_SI_spacer,tau_y_spacer,rho_spacer);   
    Frl(k)     = unified_friction_pipe(mrl_d,q_avgm,hssp+mrl_hoz,PV_mud,YP_SI_mud,tau_y_mud,rho_mud); 
    Pspm_out(k)= Frl(k) +  Grl(k);
    DPspm(k)   = Pspm_out(k) - Pspm_in(k);
    H(k)       = DPspm(k)/(np*rho_mud*g);    
    %Pdha(k)    = Fa(k) + Ga(k);    
      
      
end
% Plots